%module Cg
%{
#include "Cg/cg.h"
#include "Cg/cgGL.h"
#include <Numeric/arrayobject.h>
%}

%init %{
  import_array();
%}

%typemap(python,in) const float * {
  PyArrayObject* array;
  
  if ( !PyArray_Check($input) ) {
    PyErr_SetString(PyExc_ValueError, "CALLE: not array!" );
    return;
  }
  
  array = (PyArrayObject*)PyArray_ContiguousFromObject( $input, PyArray_FLOAT, 1, 1 );

  if ( array->descr->type_num != PyArray_FLOAT ) {
    PyErr_SetString(PyExc_ValueError, "CALLE: not float array!" );
    return;
  }

  $1 = (float*)array->data;
}

%typemap(python,in) const double * {
  PyArrayObject* array;
  
  if ( !PyArray_Check($input) ) {
    PyErr_SetString(PyExc_ValueError, "CALLE: not array!" );
    return;
  }
  
  array = (PyArrayObject*)PyArray_ContiguousFromObject( $input, PyArray_DOUBLE, 1, 2 );
  
  if ( array->descr->type_num != PyArray_DOUBLE ) {
    PyErr_SetString(PyExc_ValueError, "CALLE: not double array!" );
    return;
  }
  
  $1 = (double*)array->data;
}

%typemap(python,in) GLenum type {
  $1 = (int) PyInt_AsLong($input);
  if ( PyErr_Occurred() ) {
    return;
  }
}

%typemap(python,in) GLsizei stride {
  $1 = (int) PyInt_AsLong($input);
  if ( PyErr_Occurred() ) {
    return;
  }
}

%typemap(python,in) GLvoid *pointer {
  PyArrayObject* array;
  
  if ( !PyArray_Check($input) ) {
    PyErr_SetString(PyExc_ValueError, "CALLE: not array!" );
    return;
  }
  
  if ( array = (PyArrayObject*)PyArray_ContiguousFromObject( $input, PyArray_DOUBLE, 1, 2 ) ) {
    $1 = (double*)array->data;
  } else if ( array = (PyArrayObject*)PyArray_ContiguousFromObject( $input, PyArray_FLOAT, 1, 2 ) ) {
    $1 = (float*)array->data;
  } else {
    PyErr_SetString(PyExc_ValueError, "CALLE: could not cast!" );
    return;
  }
}


%include "types.i"
%include "constants.i"
%include "funcs.i"


/* Error handling function */
%{
/* From PyOpenGL 
 * 
 * In the GLUT section, PyOpenGL uses a similar method to
 * handle callback functions.  I reviewed their source,
 * made minor changes, and did this my way. -CM
 */
#define PyErr_XPrint() if (PyErr_Occurred()) PyErr_Print()

static PyObject *errorCallback = NULL;

void _internalErrorCallback() 
{
	if (errorCallback && errorCallback != Py_None) {
		PyObject *result = PyObject_CallFunction(errorCallback, NULL);
		Py_XDECREF(result);
		PyErr_XPrint();
	}
}

void _cgSetErrorCallback(PyObject *func)
{
	PyObject *old = errorCallback;

	errorCallback = func;

	Py_XINCREF(func);
	cgSetErrorCallback((func == Py_None) ? NULL : _internalErrorCallback);
	Py_XDECREF(old);
}

%}
%name(cgSetErrorCallback) void _cgSetErrorCallback(PyObject * pyfunc);

%{
PyObject *_cgGetErrorCallback()
{
	return errorCallback;
}
%}
%name(cgGetErrorCallback) PyObject *_cgGetErrorCallback();

